## Access control

Access control is implemented through capabilities.
Capabilities define what users can or cannot do. They are divided into two categories:

- **User capabilities**: define user access control; these are stored in the user config
- **VM access capabilities**: define access to VMs; these are stored in the VM config

### User capabilities

The various operations a user can or cannot do are defined by her/his capabilities.

The table below lists all potential capabilities associated to a user:

| Capability        | Description                                       |
|---                |---                                                |
| USER_CREATE       | Can create a user                                 |
| USER_DESTROY      | Can destroy a user                                |
| USER_SET_CAPS     | Can change a user's capabilities                  |
| USER_UNLOCK       | Can unlock a user's account                       |
| USER_RESETPWD     | Can reset a user's password                       |
| USER_LIST         | Can list all users                                |
| VM_CREATE         | Can create a VM                                   |
| VM_DESTROY_ANY    | Can destoy **ANY** VM                             |
| VM_EDIT_ANY       | Can edit **ANY** VM                               |
| VM_START_ANY      | Can start **ANY** VM                              |
| VM_STOP_ANY       | Can kill/shutdown **ANY** VM                      |
| VM_REBOOT_ANY     | Can Reboot **ANY** VM                             |
| VM_LIST_ANY       | Can list **ANY** VM                               |
| VM_ATTACH_ANY     | Can attach to **ANY** VM                          |
| VM_READFS_ANY     | Can export files from **ANY** VM                  |
| VM_WRITEFS_ANY    | Can import files into **ANY** VM                  |
| VM_SET_ACCESS     | Can change (edit or delete) a VM's access         |
| VM_SET_ACCESS_ANY | Can change (edit or delete) any VM's access       |
| TPL_CREATE        | Can create a template                             |
| TPL_EDIT          | Can edit a template                               |
| TPL_EDIT_ANY      | Can edit **ANY** template                         |
| TPL_DESTROY       | Can destroy a template                            |
| TPL_DESTROY_ANY   | Can destroy **ANY** template                      |
| TPL_LIST          | Can list public or owned templates                |
| TPL_LIST_ANY      | Can list **ANY** template                         |
| TPL_READFS        | Can export files from a public or owned templates |
| TPL_READFS_ANY    | Can export files from **ANY** template            |
| EXAM_ATTACH       | Can attach to a VM for which creds are known      |

### VM access capabilities

What a user can or cannot do **with a VM** is defined by the capabilities stored in the VM for this very user.

The table below lists all potential capabilities granted to a user for a given VM.
These capabilities are called "VM access capabilities":

| Capability    | Description                                                        |
|---            |---                                                                 |
| VM_SET_ACCESS | User can add/change access to the (running or stopped) VM          |
|               | VM_SET_ACCESS **must also be present** in the user's capabilities! |
| VM_DESTROY    | User can destroy the (stopped) VM                                  |
| VM_EDIT       | User can edit the (running or stopped) VM                          |
| VM_START      | User can start the (stopped) VM                                    |
| VM_STOP       | User can kill/shutdown the (running) VM                            |
| VM_REBOOT     | User can reboot the (running) VM                                   |
| VM_LIST       | User can list the VM's meta-data                                   |
| VM_ATTACH     | User can attach to the (running) VM                                |
| VM_READFS     | User can export files from the (stopped) VM                        |
| VM_WRITEFS    | User can import files into the (stopped) VM                        |

### IMPORTANT

- When a user creates a VM, she/he is this VM's owner
  - the owner is granted all VM access capabilities
- No other users is granted any access to the created VM. However, the VM owner can add any access type to any users they like
