package cmdTemplate

import (
	"errors"
	"strings"

	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/params"
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/nexush/utils"
)

type Edit struct {
	Name string
}

func (cmd *Edit) GetName() string {
	return cmd.Name
}

func (cmd *Edit) GetDesc() []string {
	return []string{
		"Edits one or more template's properties: name, access.",
		"Requires TPL_EDIT or TPL_EDIT_ANY user capability."}
}

func (cmd *Edit) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: " + cmd.GetName() + " ID [name=\"new name\"] [access=private/public]")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("ID      ID of the template to edit.")
	const usage string = `Parameters that can be changed (at least one must be specified):
name    Name of the template.
access  Access type, either "private" or "public".`
}

func (cmd *Edit) Run(client *nc.NexusClient, args []string) int {
	argc := len(args)
	if argc < 2 {
		cmd.PrintUsage()
		return 1
	}

	p, err := cmd.parseArgs(args[1:])
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}

	if p == nil {
		cmd.PrintUsage()
		return 1
	}

	tplID := args[0]
	template, err := client.TemplateEdit(tplID, *p)
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}

	templateStr, err := template.String()
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}
	u.Println("Template \"" + tplID + "\" successfully edited:")
	u.Println(templateStr)
	return 0
}

func (cmd *Edit) parseArgs(args []string) (*params.TplEdit, error) {
	p := &params.TplEdit{}
	atLeastOneArg := false

	getStringVal := func(s string, prefix string) string {
		if strings.HasPrefix(s, prefix) {
			parts := strings.Split(s, prefix)
			return parts[1]
		}
		return ""
	}

	for _, arg := range args {
		s := getStringVal(arg, "name=")
		if s != "" {
			p.Name = s
			atLeastOneArg = true
			continue
		}
		s = getStringVal(arg, "access=")
		if s != "" {
			p.Access = s
			atLeastOneArg = true
			continue
		}
		return nil, errors.New("Invalid argument: " + arg)
	}

	if atLeastOneArg {
		return p, nil
	} else {
		return nil, nil
	}
}
