package main

import (
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/cleaner"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/config"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/exec"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/logger"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/router"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/users"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/utils"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/vms"
)

var log = logger.GetInstance()

func main() {
	conf := config.GetInstance()

	// Initialize the RNG' seed to the current time.
	utils.RandInit()

	if err := exec.CheckQemuSystem(); err != nil {
		log.Fatal(err)
	}
	if err := exec.CheckQemuImg(); err != nil {
		log.Fatal(err)
	}
	if err := exec.CheckGuestfish(); err != nil {
		log.Fatal(err)
	}

	err := users.InitUsers()
	if err != nil {
		log.Fatal(err.Error())
	}

	// Templates are required by VMs hence they're loaded first.
	err = vms.InitTemplates()
	if err != nil {
		log.Fatal(err.Error())
	}

	err = vms.InitVMs()
	if err != nil {
		log.Fatal(err.Error())
	}

	cleaner.Start()

	log.Info("API port: ", conf.Core.APIDefaultPort)
	log.Info("Spice port range: ", conf.Core.VMSpiceMinPort, "-", conf.Core.VMSpiceMaxPort)
	log.Info("Tmp directory: ", conf.Core.TmpDir)

	log.Info("Max upload size: ", conf.Limits.MaxUploadSize)
	log.Info("KSM RAM saving: ", conf.Limits.KsmRamSaving*100, "%")
	log.Info("RAM usage limit: ", conf.Limits.RamUsageLimit*100, "%")

	log.Info("Max login attemps: ", conf.Auth.MaxLoginAttempts)
	log.Info("Account lock time: ", conf.Auth.LockedTimeInHours, " hours")
	log.Info("Access token expiration time: ", conf.Auth.AccessTokenExpirationTimeInHours, " hours")

	router.New().Start(conf.Core.APIDefaultPort)
}
