package users

import (
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/caps"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/params"

	"github.com/go-playground/validator/v10"
)

type User params.UserWithPwd

var dummyUser = User{}

// Creates an empty user.
func NewEmptyUser() *User {
	return &User{Email: "", FirstName: "", LastName: "", Caps: make(caps.Capabilities), Pwd: ""}
}

// Checks that the User structure's fields are valid.
func (user *User) Validate() error {
	// Checks the capabilities are valid
	if err := caps.ValidateUserCaps(user.Caps); err != nil {
		return err
	}

	return validator.New(validator.WithRequiredStructEnabled()).Struct(user)
}

// Returns true if user has the specified capability.
func (user *User) HasCapability(capability string) bool {
	_, exists := user.Caps[capability]
	return exists
}

func (user *User) GetVMAccessCapabilities() []string {
	caps := []string{}
	for key := range user.Caps {
		caps = append(caps, key)
	}
	return caps

	// capabilities := []string{}
	// for cap, _ := range caps.GetVMAccessCapsNames() {
	//     _, exists := user.Caps[cap]
	//     if exists {
	//         capabilities = append(capabilities, cap)
	//     }
	// }
	// return capabilities
}
